def parseBitsoTrade(response):
    return {
        'book': response['book'],
        'created_at': response['created_at'],
        'amount': float(response['amount']),
        'maker_side': response['maker_side'],
        'price': float(response['price']),
        'tid': response['tid']
    }

def parseBitsoTradeArray(response):
    result = []
    for datum in response:
        result.append(parseBitsoTrade(datum))
    return result

def parseBitsoTradeDetails(response):
    return {
        'book': response['book'],
        'created_at': response['created_at'],
        'maker_side': response['maker_side'],
        'price': float(response['price']),
        'tid': response['tid'],
        'fees_amount': float(response['fees_amount']),
        'fees_currency': response['fees_currency'],
        'major': float(response['major']),
        'major_currency': response['major_currency'],
        'minor': float(response['minor']),
        'minor_currency': response['minor_currency'],
        'side': response['side'],
        'oid': response['oid']
    }

def parseBitsoTradeDetailsArray(response):
    result = []
    for datum in response:
        result.append(parseBitsoTradeDetails(datum))
    return result


import math

class BinanceStrategyActuator:
    def __init__(self, connection, config):
        self.connection = connection
        self.config = config
    
    def canBeUsed(self, orderBook):
        pass
    
    def act(self, orderBook):
        pass
    
    def adjustPrice(self, price):
        adjustedPrice = math.floor(price / self.config['tickSize']) * self.config['tickSize']
        return round(adjustedPrice, self.config['decimals'])
    
    def adjustQuantity(self, quantity):
        return round(quantity, self.config['quantityDecimals'])
    
    def findOrder(self, orderId):
        orders = self.connection.get_all_orders(symbol=self.config['symbol'], orderId=orderId, recvWindow=2000)
        for order in orders:
            if str(order['orderId']) == str(orderId):
                return order
        return None
    
    def isFilled(self, order):
        return order['status'] == 'PARTIALLY_FILLED' or order['status'] == 'FILLED'
    
    def operate(self, side, price, quantity):
        order = self.connection.new_order(
            symbol=self.config['symbol'],
            side=side,
            type="LIMIT",
            timeInForce="GTC",
            price=self.adjustPrice(price),
            quantity=self.adjustQuantity(quantity)
        )
        order['price'] = float(order['price'])
        order['origQty'] = float(order['origQty'])
        order['executedQty'] = float(order['executedQty'])
        order['cummulativeQuoteQty'] = float(order['cummulativeQuoteQty'])
        return order


from cetus_tools.binance.strategy.control.OrderAdaptor import OrderAdaptor

class ProfitAdaptor(OrderAdaptor):
    def init(self, order):
        order['profit']['adaptionPrice'] = order['operation']['price'] + (0.5 * (order['profit']['price'] - order['operation']['price']))
        order['profit']['adaptionSize'] = 0.6 * (order['profit']['price'] - order['operation']['price'])
        order['profit']['adaptionEnabled'] = False

    def adapt(self, orderBook, order):
        top = orderBook['topBid'] if order['operation']['side'] == 'BUY' else orderBook['topAsk']
        if top != None and self.isAdaptionEvent(top, order):
            order['profit']['adaptionEnabled'] = True
            order['profit']['adaptionPrice'] = top['price']
            return order['profit']['adaptionPrice'] - order['profit']['adaptionSize']
        return order['profit']['adaptionPrice'] - order['profit']['adaptionSize'] if order['profit']['adaptionEnabled'] == True else order['loss']['price']
    
    def isAdaptionEvent(self, top, order):
        return top['price'] > order['profit']['adaptionPrice'] if order['operation']['side'] == 'BUY' else top['price'] < order['profit']['adaptionPrice']


import math
import requests
from datetime import datetime

class LogManager:
    def __init__(self, url):
        self.url = url
    
    def register(self, strategyId, details):
        response = (requests.post(self.url + '/v1/executions', json={
            'timestamp': math.floor(1000 * datetime.timestamp(datetime.now())),
            'strategyId': strategyId,
            'details': details
        })).json()
        return response['payload']['executionId']
    
    def sendExecutionLog(self, executionId, action, details):
        return (requests.post(self.url + '/v1/executions/' + executionId + '/logs', json={
            'timestamp': math.floor(1000 * datetime.timestamp(datetime.now())),
            'action': action,
            'details': details
        })).json()


def parseBitsoOrderArray(values):
    result = []
    for value in values:
        result.append({
            'book': value['book'],
            'price': float(value['price']),
            'amount': float(value['amount']),
            'oid': value['oid'] if 'oid' in value.keys() else None
        })
    return result

def parseBitsoOrderBook(response):
    return {
        'updated_at': response['updated_at'],
        'sequence': int(response['sequence']),
        'bids': parseBitsoOrderArray(response['bids']),
        'asks': parseBitsoOrderArray(response['asks'])
    }


import datetime
import math
import time

class SetIcapRatesManager:
    def __init__(self, apiManager):
        self.apiManager = apiManager
    
    def getSetIcapRate(self, prices):
        if self.isWorkingHours():
            rates = self.apiManager.getRates()
            for rate in rates:
                if self.isValidRate(rate):
                    return self.buildRatePrices(rate, prices)
        return None
    
    def isValidRate(self, rate):
        return (rate['symbol'] == 'USDCOP') and (rate['source'] == 'CMA') and('bid' in rate.keys()) and ('ask' in rate.keys())
    
    def isWorkingHours(self):
        date = datetime.datetime.now(datetime.timezone.utc)
        return date.hour >= 13 and date.hour < 22 and date.weekday() < 5 
    
    def buildRatePrices(self, rate, prices):
        rate['timestamp'] = math.floor(1000 * time.time())
        for price in prices:
            if price['key'] in rate.keys():
                rate[price['name']] = max(rate[price['key']] - price['spread'], 0)
        return rate


import requests
import hashlib
import json
import math
import time
import hmac

def convert_float_like_integers(data):
    if isinstance(data, dict):
        return {k: convert_float_like_integers(v) for k, v in data.items()}
    elif isinstance(data, list):
        return [convert_float_like_integers(item) for item in data]
    elif isinstance(data, float) and data.is_integer():
        return int(data)
    else:
        return data

class StrategyApiManager:
    def __init__(self, keys, apiUrl):
        self.apiUrl = apiUrl
        self.keys = keys
    
    def sign(self, method, path, body):
        nonce = math.floor(1000 * time.time())
        message = str(nonce) + method + path.replace(" ", "%20") + json.dumps(convert_float_like_integers(body), separators=(',', ':'))
        signature = hmac.new(bytes(self.keys['secretKey'], 'utf-8'), msg=bytes(message, 'utf-8'), digestmod=hashlib.sha256).hexdigest()
        return { "Authorization": "cetus " + self.keys['publicKey'] + ":" + str(nonce) + ":" + signature }
    
    def getStrategies(self, name, pair, account):
        url = "/v1/strategies?name=" + name + "&pair=" + pair + "&account=" + account
        signature = self.sign("GET", url, {})
        return (requests.get(self.apiUrl + url, headers=signature)).json()['result']
    
    def updateStrategy(self, strategy):
        url = "/v1/strategies/" + str(strategy['id'])
        payload = { 'memory': strategy['memory'] }
        signature = self.sign("PUT", url, payload)
        return requests.put(self.apiUrl + url, headers=signature, json=payload).json()
    
    def getWallets(self, address, network):
        url = "/v1/kyc/wallets?address=" + address + "&network=" + network
        signature = self.sign("GET", url, {})
        return requests.get(self.apiUrl + url, headers=signature).json()['result']
    
    def createWallet(self, address, network):
        url = "/v1/kyc/wallets"
        payload = { 'address': address, 'network': network }
        signature = self.sign("POST", url, payload)
        return requests.post(self.apiUrl + url, headers=signature, json=payload).json()
    
    def createWalletActivity(self, walletId, activityType, details):
        url = "/v1/kyc/activities"
        payload = { 'walletId': walletId, 'type': activityType, 'details': details, 'timestamp': math.floor(1000 * time.time()) }
        signature = self.sign("POST", url, payload)
        return requests.post(self.apiUrl + url, headers=signature, json=payload).json()
    
    def getRates(self):
        url = "/v1/rates"
        signature = self.sign("GET", url, {})
        return (requests.get(self.apiUrl + url, headers=signature)).json()['rates']
    
    def createOrder(self, strategyId, order):
        url = "/v1/strategies/orders"
        payload = { 'strategyId': strategyId, 'order': order }
        signature = self.sign("POST", url, payload)
        return requests.post(self.apiUrl + url, headers=signature, json=payload).json()
    
    def getBlockchainTransaction(self, txhash, network):
        url = "/v1/blockchain/transactions?txhash=" + txhash + "&network=" + network
        signature = self.sign("GET", url, {})
        return requests.get(self.apiUrl + url, headers=signature).json()['result']


def calculateDecimals(tickSize):
    cnt = 0
    while tickSize < 1:
        tickSize *= 10
        cnt = cnt +1
    return cnt

def parseBitsoBookFeesRate(response):
    return {
        'maker': float(response['maker']),
        'taker': float(response['taker']),
        'volume': float(response['maker']) if 'volume' in response.keys() else None
    }

def parseBitsoBookFeesRateArray(response):
    result = []
    for value in response:
        result.append(parseBitsoBookFeesRate(value))
    return result

def parseBitsoBook(response):
    return {
        'book': response['book'],
        'default_chart': response['default_chart'],
        'minimum_price': float(response['minimum_price']),
        'maximum_price': float(response['maximum_price']),
        'minimum_value': float(response['minimum_value']),
        'maximum_value': float(response['maximum_value']),
        'minimum_amount': float(response['minimum_amount']),
        'maximum_amount': float(response['maximum_amount']),
        'tick_size': float(response['tick_size']),
        'decimals': calculateDecimals(float(response['tick_size'])),
        'fees': {
            'flat_rate': parseBitsoBookFeesRate(response['fees']['flat_rate']),
            'structure': parseBitsoBookFeesRateArray(response['fees']['structure'])
        }
    }

def parseBitsoBookArray(response):
    result = []
    for datum in response:
        result.append(parseBitsoBook(datum))
    return result

def findBitsoBook(pairs, books):
    result = {}
    for book in books:
        if book['book'] in pairs:
            result[book['book']] = book
    return result


from cetus_tools.binance.strategy.control.OrderAdaptor import OrderAdaptor

class ForceClosingAdaptor(OrderAdaptor):
    def init(self, order):
        order['loss']['forceClosingSize'] = 0.05 * (order['profit']['price'] - order['operation']['price'])
        order['loss']['forceClosingEnabled'] = False

    def adapt(self, orderBook, order):
        top = orderBook['topBid'] if order['operation']['side'] == 'BUY' else orderBook['topAsk']
        return top['price'] - order['loss']['forceClosingSize'] if top != None and order['loss']['forceClosingEnabled'] == True else order['loss']['price']


def parseBitsoOrderDetails(response):
    return {
        'book': response['book'],
        'unfilled_amount': float(response['unfilled_amount']),
        'original_amount': float(response['original_amount']),
        'original_value': float(response['original_value']),
        'created_at': response['created_at'],
        'updated_at': response['updated_at'],
        'oid': response['oid'],
        'side': response['side'],
        'status': response['status'],
        'time_in_force': response['time_in_force'],
        'type': response['type'],
        'origin_id': response['origin_id'] if 'origin_id' in response.keys() else None,
        'price': float(response['price']) if 'price' in response.keys() else None,
        'stop': float(response['stop']) if 'stop' in response.keys() else None,
        'triggered_at': response['triggered_at'] if 'triggered_at' in response.keys() else None
    }

def parseBitsoOrderDetailsArray(response):
    result = []
    for datum in response:
        result.append(parseBitsoOrderDetails(datum))
    return result


import time
import requests

from cetus_tools.BrokerManager import BrokerManager
from cetus_tools.bitso.api.tools.book import findBitsoBook
from cetus_tools.bitso.api.tools.balance import findBalances

class BitsoManager(BrokerManager):
    def __init__(self, connection):
        super().__init__('bitso')
        self.connection = connection
    
    def getOrderBook(self, book):
        pass
    
    def getTopPrices(self, book):
        books = findBitsoBook([book['book']], self.connection.getAvailableBookList())
        orderBook = self.connection.getOrderBook(books[book['book']], None)
        return {
            'book': book['book'],
            'bid': orderBook['bids'][0]['price'],
            'bidSize': orderBook['bids'][0]['amount'],
            'ask': orderBook['asks'][0]['price'],
            'askSize': orderBook['asks'][0]['amount']
        }
    
    def getBalances(self, book):
        return findBalances(book['assets'], self.connection.getBalances())
    
    def operate(self, book, side, price, quantity):
        order = self.connection.placeOrder({
            'timestamp': 1000 * time.time(),
            'book': book['book'],
            'side': side,
            'type': 'limit',
            'status': 'placed',
            'price': self.adjustPrice(price, book),
            'major': self.adjustQuantity(quantity, book)
        })
        if order != None and 'oid' in order.keys():
            order = self.connection.lookUpOrder(order['oid'])[0]
            order['symbol'] = book['book']
            order['orderId'] = order['oid']
            order['timestamp'] = 1000 * time.time()
            order['source'] = self.source
            order['origQty'] = order['original_amount']
            order['executedQty'] = order['original_amount'] - order['unfilled_amount']
            order['cummulativeQuoteQty'] = order['price'] * order['executedQty']
            return order
        return None
    
    def isFilledOrder(self, order):
        return order['status'] == 'completed'
    
    def isPartiallyFilledOrder(self, order):
        return order['status'] == 'partially filled'
    
    def isCancelledOrder(self, order):
        return order['status'] == 'cancelled'
    
    def cancelOrder(self, order):
        return self.connection.cancelOrder(order['oid'])
    
    def findOrder(self, order):
        return self.connection.lookUpOrder(order['oid'])
    
    def updateOrder(self, order):
        update = self.connection.lookUpOrder(order['oid'])[0] if 'oid' in order.keys() else None
        if update != None:
            order['unfilled_amount'] = update['unfilled_amount']
            order['status'] = update['status']
            order['executedQty'] = update['original_amount'] - update['unfilled_amount']
            order['cummulativeQuoteQty'] = order['price'] * order['executedQty']


import time
import hmac
import json
import hashlib
import requests

from cetus_tools.bitso.api.tools.book import parseBitsoBookArray
from cetus_tools.bitso.api.tools.trade import parseBitsoTradeArray
from cetus_tools.bitso.api.tools.orderBook import parseBitsoOrderBook
from cetus_tools.bitso.api.tools.balance import parseBitsoBalanceArray
from cetus_tools.bitso.api.tools.order import parseBitsoOrderDetailsArray
from cetus_tools.bitso.api.tools.trade import parseBitsoTradeDetailsArray

class BitsoApiConnection:
    def __init__(self, config):
        self.config = config
    
    def sign(self, method, path, body):
        nonce = int(time.time() * 1000)
        message = bytes(str(nonce) + method + path + (json.dumps(body) if body != None else ""), 'utf-8')
        signature = hmac.new(self.config['apiSecret'], msg=message, digestmod=hashlib.sha256).hexdigest()
        return { 'Authorization': 'Bitso ' +  self.config['apiKey'] + ':' + str(nonce) + ':' + signature }
    
    def getBalances(self):
        signature = self.sign('GET', '/api/v3/balance/', None)
        response = requests.get(self.config['url'] + '/api/v3/balance/', headers=signature).json()
        return parseBitsoBalanceArray(response['payload']['balances'])
    
    def getAvailableBookList(self):
        signature = self.sign('GET', '/api/v3/available_books/', None)
        response = requests.get(self.config['url'] + '/api/v3/available_books/', headers=signature).json()
        return parseBitsoBookArray(response['payload'])
    
    def getOrderBook(self, book, aggregate):
        url = '/api/v3/order_book/?book=' + book['book'] + '&aggregate=' + str(aggregate if aggregate != None else True)
        signature = self.sign('GET', url, None)
        response = requests.get(self.config['url'] + url, headers=signature).json()
        return parseBitsoOrderBook(response['payload'])
    
    def placeOrder(self, body):
        signature = self.sign('POST', '/api/v3/orders/', body)
        response = requests.post(self.config['url'] + '/api/v3/orders/', json=body, headers=signature).json()
        return response['payload'] if response['success'] else None
    
    def lookUpOrder(self, oid):
        url = '/api/v3/orders/' + oid + '/'
        signature = self.sign('GET', url, None)
        response = requests.get(self.config['url'] + url, headers=signature).json()
        return parseBitsoOrderDetailsArray(response['payload']) if response['success'] else []
    
    def getUserOpenOrders(self, book):
        url = '/api/v3/open_orders/?book=' + book['book']
        signature = self.sign('GET', url, None)
        response = requests.get(self.config['url'] + url, headers=signature).json()
        return parseBitsoOrderDetailsArray(response['payload']) if response['success'] else []
    
    def cancelOrder(self, oid):
        url = '/api/v3/orders/' + oid + '/'
        signature = self.sign('DELETE', url, None)
        response = requests.delete(self.config['url'] + url, headers=signature).json()
        return response['payload'] if response['success'] else []
    
    def getTrades(self, book):
        url = '/api/v3/trades/?book=' + book['book'] + ('&limit=' + str(book['limit']) if 'limit' in book.keys() else '') + ('&maker=' + str(book['maker']) if 'maker' in book.keys() else '') + ('&sort=' + str(book['sort']) if 'sort' in book.keys() else '')
        signature = self.sign('GET', url, None)
        response = requests.get(self.config['url'] + url, headers=signature).json()
        return parseBitsoTradeArray(response['payload']) if response['success'] else []
    
    def getOrderTrades(self, oid):
        try:
            url = '/api/v3/order_trades/' + oid + '/'
            signature = self.sign('DELETE', url, None)
            response = requests.delete(self.config['url'] + url, headers=signature).json()
            return parseBitsoTradeDetailsArray(response['payload']) if response['success'] else []
        except:
            print("> error getting trades")
            return []

    def getFees(self):
        signature = self.sign('GET', '/api/v3/fees/', None)
        response = requests.get(self.config['url'] + '/api/v3/fees/', headers=signature).json()
        return response
    
    def getTicker(self, book):
        url = '/api/v3/ticker/?book=' + book['book']
        signature = self.sign('GET', url, None)
        response = requests.get(self.config['url'] + url, headers=signature).json()
        return response


import math

class BrokerManager:
    def __init__(self, source):
        self.source = source

    def getOrderBook(self, book):
        pass

    def getTopPrices(self, connection, book):
        pass
    
    def getBalances(self, book):
        pass
    
    def operate(self, book, side, price, quantity):
        pass
    
    def isFilledOrder(self, order):
        pass
    
    def isPartiallyFilledOrder(self, order):
        pass
    
    def isCancelledOrder(self, order):
        pass
    
    def cancelOrder(self, order):
        pass
    
    def updateOrder(self, order):
        pass
    
    def findOrder(self, order):
        pass
    
    def adjustPrice(self, price, book):
        adjustedPrice = math.floor(price / book['tickSize']) * book['tickSize']
        return round(adjustedPrice, book['decimals'])
    
    def adjustQuantity(self, quantity, book):
        return round(quantity, book['quantityDecimals'])


from cetus_tools.binance.event.OrderBookProcessing import isOrderBookEvent, parseOrderBook, parseOrderArray, isApiOrderBookEvent, parseApiOrderBook
from cetus_tools.binance.strategy.BinanceStrategy import BinanceStrategy
from cetus_tools.binance.strategy.BinanceStrategyActuator import BinanceStrategyActuator
from cetus_tools.binance.strategy.control.OrderAdaptor import OrderAdaptor
from cetus_tools.binance.strategy.control.SetIcapRatesManager import SetIcapRatesManager

from cetus_tools.binance.strategy.control.adaptors.ForceClosingAdaptor import ForceClosingAdaptor
from cetus_tools.binance.strategy.control.adaptors.LossAdaptor import LossAdaptor
from cetus_tools.binance.strategy.control.adaptors.ProfitAdaptor import ProfitAdaptor
from cetus_tools.binance.strategy.control.adaptors.SecureProfitAdaptor import SecureProfitAdaptor

from cetus_tools.binance.strategy.control.crud.StrategyCRUD import findOrCreateStrategy, findStrategyRequests, updateStrategyRequest, updateStrategy

from cetus_tools.bitso.api.BitsoApiConnection import BitsoApiConnection

from cetus_tools.binance.BinanceManager import BinanceManager
from cetus_tools.bitso.BitsoManager import BitsoManager
from cetus_tools.BrokerManager import BrokerManager

from cetus_tools.cetus.LogManager import LogManager
from cetus_tools.cetus.StrategyApiConnection import StrategyApiManager
from cetus_tools.cetus.data.BinanceMarketDataManager import BinanceMarketDataManager
from cetus_tools.cetus.data.MarketDataServiceManager import MarketDataServiceManager

from cetus_tools.telegram.TelegramChannelManager import TelegramChannelManager


import time

from cetus_tools.binance.event.OrderBookProcessing import parseOrderArray
from cetus_tools.BrokerManager import BrokerManager

class BinanceManager(BrokerManager):
    def __init__(self, connection):
        super().__init__('binance')
        self.connection = connection
    
    def getOrderBook(self, book):
        binanceBook = self.connection.depth(book['book'], limit=10)
        return {
            'book': book['book'],
            'payload': {
                'timestamp': int(1000 * time.time()),
                'bids': parseOrderArray(binanceBook['bids']),
                'asks': parseOrderArray(binanceBook['asks']),
            }
        }
    
    def getTrades(self, book):
        result = { 'book': book['book'], 'payload': [] }
        for trade in self.connection.trades(book['book']):
            if book['lastTradeId'] < trade['id']:
                result['payload'].append({
                    'id': trade['id'],
                    'timestamp': trade['time'],
                    'price': float(trade['price']),
                    'qty': float(trade['qty']),
                    'quoteQty': float(trade['quoteQty']),
                    'isBuyerMaker': trade['isBuyerMaker'],
                    'isBestMatch': trade['isBestMatch']
                })
                book['lastTradeId'] = trade['id']
        return result
    
    def getTopPrices(self, book):
        binanceBook = self.connection.book_ticker(book['book'])
        return {
            'book': book['book'],
            'bid': float(binanceBook['bidPrice']),
            'bidSize': float(binanceBook['bidQty']),
            'ask': float(binanceBook['askPrice']),
            'askSize': float(binanceBook['askQty'])
        }
    
    def getBalances(self, book):
        return self.findBinanceBalances(book['assets'], self.connection.account()['balances'])
    
    def findBinanceBalances(self, assets, balances):
        result = {}
        for balance in balances:
            if balance['asset'].lower() in assets:
                result[balance['asset'].lower()] = {
                    'currency': balance['asset'].lower(),
                    'available': float(balance['free']),
                    'locked': float(balance['locked'])
                }
        return result
    
    def operate(self, book, side, price, quantity):
        order = self.connection.new_order(
            symbol=book['book'],
            side=side,
            type="LIMIT",
            timeInForce="GTC",
            price=self.adjustPrice(price, book),
            quantity=self.adjustQuantity(quantity, book)
        )
        order['source'] = self.source
        order['timestamp'] = 1000 * time.time()
        order['price'] = float(order['price'])
        order['origQty'] = float(order['origQty'])
        order['executedQty'] = float(order['executedQty'])
        order['cummulativeQuoteQty'] = float(order['cummulativeQuoteQty'])
        return order
    
    def isFilledOrder(self, order):
        return order['status'] == 'FILLED'
    
    def isPartiallyFilledOrder(self, order):
        return order['status'] == 'PARTIALLY_FILLED'
    
    def isCancelledOrder(self, order):
        return order['status'] == 'CANCELED'
    
    def cancelOrder(self, order):
        self.connection.cancel_order(symbol=order['symbol'], orderId=order['orderId'])
    
    def updateOrder(self, order):
        update = self.findOrder(order)
        if update != None:
            order['status'] = update['status']
            order['executedQty'] = float(update['executedQty'])
            order['cummulativeQuoteQty'] = float(update['cummulativeQuoteQty'])
    
    def findOrder(self, order):
        return self.connection.get_order(symbol=order['symbol'], orderId=order['orderId'])



import requests

def findOrCreateStrategy(strategyCrudUrl, definition):
    response = (requests.get(strategyCrudUrl + '/v1/strategies?name=' + definition['name'] + '&tag=' + definition['tag'])).json()
    if len(response['result']) == 0:
        response = (requests.post(strategyCrudUrl + '/v1/strategies', json=definition)).json()
        return response['items'][0]
    return response['result'][0]

def findStrategyRequests(strategyCrudUrl, strategyId):
    try:
        response = (requests.get(strategyCrudUrl + '/v1/requests?strategyId=' + str(strategyId) + '&status=SUBMITTED,PLACED,FILLED,FORCED_CLOSING')).json()
        return response['result']
    except:
        return []

def updateStrategyRequest(strategyCrudUrl, definition):
    response = requests.put(strategyCrudUrl + '/v1/requests/' + str(definition['strategyRequestId']), json={
        'status': definition['status'],
        'details': definition['details']
    })

def updateStrategy(strategyCrudUrl, definition):
    response = requests.put(strategyCrudUrl + '/v1/strategies/' + str(definition['strategyId']), json={
        'config': definition['config'],
        'memory': definition['memory']
    })


from cetus_tools.binance.strategy.control.OrderAdaptor import OrderAdaptor

class LossAdaptor(OrderAdaptor):
    def init(self, order):
        order['loss']['adaptionPrice'] = order['operation']['price'] - (0.5 * (order['operation']['price'] - order['loss']['price']))
        order['loss']['adaptionSize'] = 0.6 * (order['operation']['price'] - order['loss']['price'])
        order['loss']['adaptionEnabled'] = False

    def adapt(self, orderBook, order):
        top = orderBook['topBid'] if order['operation']['side'] == 'BUY' else orderBook['topAsk']
        if top != None:
            if order['loss']['adaptionEnabled'] == False:
                order['loss']['adaptionEnabled'] = top['price'] < order['loss']['adaptionPrice'] if order['operation']['side'] == 'BUY' else top['price'] > order['loss']['adaptionPrice']
            elif self.isAdaptionEvent(top, order):
                order['loss']['adaptionPrice'] = top['price']
                return order['loss']['adaptionPrice'] - order['loss']['adaptionSize']
        return order['loss']['adaptionPrice'] - order['loss']['adaptionSize'] if order['loss']['adaptionEnabled'] == True else order['loss']['price']
    
    def isAdaptionEvent(self, top, order):
        return top['price'] > order['loss']['adaptionPrice'] if order['operation']['side'] == 'BUY' else top['price'] < order['loss']['adaptionPrice']


#!/usr/bin/env python
import time

def isOrderBookEvent(data):
    return ('e' in data.keys()) and (data['e'] == 'depthUpdate') and len(data['b']) > 0 and len(data['a']) > 0

def parseOrderBook(data):
    bids = parseOrderArray(data['b'])
    asks = parseOrderArray(data['a'])
    return {
        'symbol': data['s'],
        'timestamp': data['T'],
        'topBid': bids[len(bids) - 1],
        'topAsk': asks[0],
        'bids': bids,
        'asks': asks
    }

def parseOrderArray(values):
    orders = []
    for value in values:
        orders.append({
            'price': float(value[0]),
            'amount': float(value[1])
        })
    return orders

def isApiOrderBookEvent(symbol, data):
    return ('bids' in data.keys()) and ('asks' in data.keys()) and len(data['bids']) > 0 and len(data['asks']) > 0

def parseApiOrderBook(symbol, data):
    bids = parseOrderArray(data['bids'])
    asks = parseOrderArray(data['asks'])
    return {
        'symbol': symbol,
        'timestamp': time.time(),
        'topBid': bids[0],
        'secondTopBid': bids[1],
        'thirdTopBid': bids[2],
        'topAsk': asks[0],
        'bids': bids,
        'asks': asks
    }


from cetus_tools.cetus.data.MarketDataServiceManager import MarketDataServiceManager
import requests

class BinanceMarketDataManager(MarketDataServiceManager):
    def __init__(self, url):
        super().__init__(url)
    
    def sendBook(self, book):
        response = (requests.post(self.url + '/v1/source/binance/spots', json=book)).json()
    
    def sendTrades(self, trades):
        response = (requests.post(self.url + '/v1/source/binance/trades', json=trades)).json()
    
    def getLastTrade(self, book):
        response = (requests.get(self.url + '/v1/source/binance/markets/' + book['book'] + '/trades/last')).json()
        return response['result']


def parseBitsoBalance(value):
    return {
        'currency': value['currency'],
        'available': float(value['available']),
        'locked': float(value['locked']),
        'total': float(value['total']),
        'pending_deposit': float(value['pending_deposit']),
        'pending_withdrawal': float(value['pending_withdrawal'])
    }

def parseBitsoBalanceArray(values):
    result = []
    for datum in values:
        result.append(parseBitsoBalance(datum))
    return result

def findBalances(currencies, balances):
    result = {}
    for balance in balances:
        if balance['currency'] in currencies:
            result[balance['currency']] = balance
    return result


import requests
import urllib.parse

class TelegramChannelManager:
    def __init__(self, url, config):
        self.url = url
        self.config = config
    
    def sendText(self, content):
        try:
            url = self.url + '/bot' + self.config['token'] + '/sendMessage?chat_id=' + self.config['channelId']
            response = requests.get(url + '&text=' + urllib.parse.quote(content.encode('utf-8')))
        except:
            print("> Telegram message could not be sent")


from cetus_tools.binance.strategy.control.OrderAdaptor import OrderAdaptor

class SecureProfitAdaptor(OrderAdaptor):
    def init(self, order):
        order['profit']['securityPrice'] = order['operation']['price'] + (0.95 * (order['profit']['price'] - order['operation']['price']))
        order['profit']['securitySize'] = 0.05 * (order['profit']['price'] - order['operation']['price'])
        order['profit']['securityEnabled'] = False

    def adapt(self, orderBook, order):
        top = orderBook['topBid'] if order['operation']['side'] == 'BUY' else orderBook['topAsk']
        if top != None and self.isAdaptionEvent(top, order):
            order['profit']['securityEnabled'] = True
            order['profit']['securityPrice'] = top['price']
            return order['profit']['securityPrice'] - order['profit']['securitySize']
        return order['profit']['securityPrice'] if order['profit']['securityEnabled'] == True else order['loss']['price']
    
    def isAdaptionEvent(self, top, order):
        return top['price'] > order['profit']['securityPrice'] if order['operation']['side'] == 'BUY' else top['price'] < order['profit']['securityPrice']


class BinanceStrategy:
    def __init__(self, connection, logger, config):
        self.connection = connection
        self.logger = logger
        self.config = config
    
    def init(self, symbol):
        pass
    
    def apply(self, orderBook):
        pass


from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='cetus_tools',
    packages=find_packages(exclude=("tests",)),
    version='0.1.9',
    description='Library for python tools used in Cetus developments',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Camilo Alaguna',
    author_email='camilo.alaguna@itrmachines.com',
    install_requires=['requests>=2.31.0'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests',
    keywords=['Cetus', 'Binance', 'Bitso', 'Telegram', 'Public API']
)